﻿Option Strict On
Imports Microsoft.Kinect
Imports System.Windows.Media.Imaging
Imports Microsoft.Speech.AudioFormat
Imports Microsoft.Speech.Recognition
Imports System.IO
Imports System.Reflection
Imports Coding4Fun.Kinect.Wpf
Class MainWindow

    Dim Kinect As KinectSensor
    Dim myScreenImageRect As Int32Rect
    Dim myDepthPixelData As Short()
    Dim myColorPixelData As Byte()

    Dim engine As SpeechRecognitionEngine
    Dim myImage As Image
    Const targetDepth As Double = 2000

    Property Human1_bitmap As WriteableBitmap
    Property Human2_bitmap As WriteableBitmap
    Property Room_Bitmap As WriteableBitmap

 

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        If KinectSensor.KinectSensors.Count = 0 Then
            MessageBox.Show("Kinectが接続されておりません。")
            Exit Sub
        End If

        Dim sentence As Choices = New Choices
        With sentence
            .Add("わかれる")
            .Add("くっつく")
            .Add("おわり")
        End With

        Dim builder As GrammarBuilder = New GrammarBuilder
        builder.Append(sentence)
        Dim myGrammer As Grammar = New Grammar(builder)
        engine = New SpeechRecognitionEngine()
        engine.LoadGrammar(myGrammer)


        AddHandler engine.SpeechRecognized, Sub(speechSender As Object, speechArgs As SpeechRecognizedEventArgs)
                                                Try
                                                    Dim confidence = speechArgs.Result.Confidence
                                                    If confidence > 0.5 Then
                                                        Select Case speechArgs.Result.Text
                                                            Case "わかれる"
                                                                Dim fullAppName As String = Assembly.GetExecutingAssembly.GetName.CodeBase
                                                                Dim fullAppPath As String = Path.GetDirectoryName(fullAppName)

                                                                Dim bmpFrame As BitmapFrame = BitmapFrame.Create(Human2_bitmap)
                                                                Using stream As FileStream = New FileStream("dummy.png", FileMode.Create)
                                                                    Dim myPng As PngBitmapEncoder = New PngBitmapEncoder
                                                                    myPng.Frames.Add(bmpFrame)
                                                                    myPng.Save(stream)
                                                                End Using
                                                                AddHandler Kinect.AllFramesReady, AddressOf Kinect_AllFramesReady

                                                                personImage.Source = New BitmapImage(New Uri(Path.Combine(fullAppPath, "dummy.png"), UriKind.Absolute))
                                                            Case "くっつく"
                                                                personImage.Source = Nothing
                                                            Case "おわり"
                                                                If Kinect Is Nothing = False Then
                                                                    If Kinect.IsRunning = True Then
                                                                        Kinect.Stop()
                                                                        Kinect.AudioSource.Stop()
                                                                        engine.RecognizeAsyncStop()
                                                                        Kinect.Dispose()
                                                                    End If
                                                                    Environment.Exit(0)
                                                                End If

                                                        End Select
                                                    End If
                                                Catch
                                                    Exit Sub
                                                End Try
                                            End Sub



        Kinect = KinectSensor.KinectSensors(0)
        Kinect.Start()

        Dim audio As KinectAudioSource = Kinect.AudioSource

        Using s As Stream = audio.Start()
            engine.SetInputToDefaultAudioDevice()
            engine.RecognizeAsync(RecognizeMode.Multiple)
        End Using

        init_kinect()

        AddHandler CompositionTarget.Rendering, Sub(renderSender As Object, renderArgs As EventArgs)
                                                    Using colorFrame As ColorImageFrame = Kinect.ColorStream.OpenNextFrame(100)
                                                        Using depthFrame As DepthImageFrame = Kinect.DepthStream.OpenNextFrame(100)
                                                            RenderScreen(colorFrame, depthFrame)
                                                        End Using
                                                    End Using
                                                End Sub
        DataContext = Me
    End Sub


    Private Sub init_kinect()
        Try
            If Kinect Is Nothing Then
                Exit Sub
            End If
            Kinect.DepthStream.Enable(DepthImageFormat.Resolution640x480Fps30)
            Kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            Kinect.SkeletonStream.Enable()

            Dim depthStream As DepthImageStream = Kinect.DepthStream
            myDepthPixelData = New Short(Kinect.DepthStream.FramePixelDataLength - 1) {}
            myColorPixelData = New Byte(Kinect.ColorStream.FramePixelDataLength - 1) {}

            Room_Bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)

            Human1_bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)
            Human2_bitmap = New WriteableBitmap(depthStream.FrameWidth, depthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)

            myScreenImageRect = New Int32Rect(0, 0, CInt(Math.Ceiling(Human2_bitmap.Width)), CInt(Math.Ceiling(Human2_bitmap.Height)))
       
        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try
    End Sub


    Private Sub Kinect_AllFramesReady(ByVal sender As Object, ByVal e As AllFramesReadyEventArgs)
        Dim allSkeletons As Skeleton()
        Using skeletonFrameData As SkeletonFrame = e.OpenSkeletonFrame()
            If skeletonFrameData Is Nothing = True Then
                Return
            End If

            allSkeletons = New Skeleton(skeletonFrameData.SkeletonArrayLength - 1) {}
            skeletonFrameData.CopySkeletonDataTo(allSkeletons)

            Dim myFirstSkeleton As Skeleton = (From skeletonResult In allSkeletons Where skeletonResult.TrackingState = SkeletonTrackingState.Tracked Select skeletonResult).FirstOrDefault()
            If myFirstSkeleton Is Nothing Then
                Return
            End If


            ScalePosition(personImage, myFirstSkeleton.Joints(JointType.HandRight))
            GetCameraPoint(myFirstSkeleton, e)


        End Using
    End Sub

    Private Sub GetCameraPoint(ByVal first As Skeleton, ByVal e As AllFramesReadyEventArgs)

        Using depth As DepthImageFrame = e.OpenDepthImageFrame()
            If depth Is Nothing Then
                Return
            End If
            Dim rightDepthPoint As DepthImagePoint = depth.MapFromSkeletonPoint(first.Joints(JointType.HandRight).Position)
            Dim rightColorPoint As ColorImagePoint = depth.MapToColorImagePoint(rightDepthPoint.X, rightDepthPoint.Y, ColorImageFormat.RgbResolution640x480Fps30)
            CameraPosition(personImage, rightColorPoint)
        End Using
    End Sub


    Private Sub CameraPosition(ByVal element As FrameworkElement, ByVal point As ColorImagePoint)
        Canvas.SetLeft(element, point.X - (CLng(element.Width) / 2))
        Canvas.SetTop(element, point.Y - (CLng(element.Height) / 2) - 50)

    End Sub


    Private Sub ScalePosition(ByVal element As FrameworkElement, ByVal joint As Joint)
        Dim scaledJoint As Joint = joint.ScaleTo(1920, 1080, 0.3F, 0.3F)
        Canvas.SetLeft(element, scaledJoint.Position.X)
        Canvas.SetTop(element, scaledJoint.Position.Y)
    End Sub

    Private Sub RenderScreen(colorFrame As ColorImageFrame, depthFrame As DepthImageFrame)
        If Kinect Is Nothing OrElse depthFrame Is Nothing OrElse colorFrame Is Nothing Then
            Return
        End If

        Dim depth As Integer = 0
        Dim depthPixelIndex As Integer
        Dim playerIndex As Integer
        Dim colorPixelIndex As Integer

        Dim colorPoint As ColorImagePoint
        Dim colorStride As Integer = colorFrame.BytesPerPixel * colorFrame.Width

        Dim screenImageStride As Integer = colorFrame.BytesPerPixel * depthFrame.Width

        Dim ImageIndex As Integer = 0
        Dim bytePlayer2 As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}
        Dim bytePlayer1 As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}
        Dim byteRoom As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {}

        depthFrame.CopyPixelDataTo(myDepthPixelData)
        colorFrame.CopyPixelDataTo(myColorPixelData)

        For depthY As Integer = 0 To depthFrame.Height - 1
            Dim depthX As Integer = 0
            While depthX < depthFrame.Width
                depthPixelIndex = depthX + (depthY * depthFrame.Width)
                playerIndex = myDepthPixelData(depthPixelIndex) And DepthImageFrame.PlayerIndexBitmask
                colorPoint = Kinect.MapDepthToColorImagePoint(depthFrame.Format, depthX, depthY, myDepthPixelData(depthPixelIndex), colorFrame.Format)
                colorPixelIndex = (colorPoint.X * colorFrame.BytesPerPixel) + (colorPoint.Y * colorStride) ' colorPixelIndex = (colorPoint.X * 4) + (colorPoint.Y * 2560)

                If playerIndex <> 0 Then
                    depth = myDepthPixelData(depthPixelIndex) >> DepthImageFrame.PlayerIndexBitmaskWidth
                    If depth > targetDepth Then
                        bytePlayer1(ImageIndex) = myColorPixelData(colorPixelIndex)
                        'Blue    
                        bytePlayer1(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                        'Green
                        bytePlayer1(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                        'Red
                        'Alpha
                        bytePlayer1(ImageIndex + 3) = &HFF
                    Else
                        bytePlayer2(ImageIndex) = myColorPixelData(colorPixelIndex)
                        'Blue    
                        bytePlayer2(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                        'Green
                        bytePlayer2(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                        'Red
                        'Alpha
                        bytePlayer2(ImageIndex + 3) = &HFF
                    End If
                Else
                    byteRoom(ImageIndex) = myColorPixelData(colorPixelIndex)
                    'Blue    
                    byteRoom(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                    'Green
                    byteRoom(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                    'Red
                    'Alpha
                    byteRoom(ImageIndex + 3) = &HFF
                End If
                depthX = depthX + 1
                ImageIndex = ImageIndex + colorFrame.BytesPerPixel
            End While
        Next

        Human1_bitmap.WritePixels(myScreenImageRect, bytePlayer1, screenImageStride, 0)
        Human2_bitmap.WritePixels(myScreenImageRect, bytePlayer2, screenImageStride, 0)
        Room_Bitmap.WritePixels(myScreenImageRect, byteRoom, screenImageStride, 0)
    End Sub


    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If Kinect Is Nothing = False Then
            If Kinect.IsRunning = True Then
                Kinect.Stop()
                engine.RecognizeAsyncStop()
                Kinect.Dispose()
            End If
        End If

    End Sub
End Class


